@@ -22,7 +22,8 @@
 #include "exm-browse-page.h"
 #include "exm-installed-page.h"
 #include "exm-detail-view.h"
-#include "exm-release-notes-dialog.h"
+#include "exm-upgrade-assistant.h"
+#include "exm-error-dialog.h"
 
 #include "local/exm-manager.h"
 #include "local/exm-extension.h"
@@ -36,13 +37,15 @@ struct _ExmWindow
     ExmManager *manager;
 
     /* Template widgets */
-    AdwHeaderBar        *header_bar;
-    GtkSwitch           *global_toggle;
-    ExmBrowsePage       *browse_page;
-    ExmInstalledPage    *installed_page;
-    AdwLeaflet          *leaflet;
-    GtkWidget           *main_view;
-    ExmDetailView       *detail_view;
+    AdwHeaderBar         *header_bar;
+    ExmBrowsePage        *browse_page;
+    ExmInstalledPage     *installed_page;
+    AdwLeaflet           *leaflet;
+    GtkWidget            *main_view;
+    ExmDetailView        *detail_view;
+    AdwViewSwitcherTitle *title;
+    AdwViewStack         *view_stack;
+    AdwToastOverlay      *toast_overlay;
 };
 
 G_DEFINE_TYPE (ExmWindow, exm_window, ADW_TYPE_APPLICATION_WINDOW)
@@ -261,6 +264,21 @@ extension_install (GtkWidget  *widget,
                                self);
 }
 
+static void
+show_page (GtkWidget  *widget,
+           const char *action_name,
+           GVariant   *param)
+{
+    ExmWindow *self;
+    char *target;
+
+    g_variant_get (param, "s", &target);
+
+    self = EXM_WINDOW (widget);
+
+    adw_view_stack_set_visible_child_name (self->view_stack, target);
+}
+
 static void
 show_view (GtkWidget  *widget,
            const char *action_name,
@@ -286,18 +304,67 @@ show_view (GtkWidget  *widget,
 }
 
 static void
-show_release_notes (GtkWidget  *widget,
-                    const char *action_name,
-                    GVariant   *param)
+show_upgrade_assistant (GtkWidget  *widget,
+                        const char *action_name,
+                        GVariant   *param)
 {
     ExmWindow *self;
 
     self = EXM_WINDOW (widget);
 
-    ExmReleaseNotesDialog *notes = exm_release_notes_dialog_new ();
-    gtk_window_set_modal (GTK_WINDOW (notes), TRUE);
-    gtk_window_set_transient_for (GTK_WINDOW (notes), GTK_WINDOW (self));
-    gtk_window_present (GTK_WINDOW (notes));
+    ExmUpgradeAssistant *assistant = exm_upgrade_assistant_new (self->manager);
+    gtk_window_set_modal (GTK_WINDOW (assistant), TRUE);
+    gtk_window_set_transient_for (GTK_WINDOW (assistant), GTK_WINDOW (self));
+    gtk_window_present (GTK_WINDOW (assistant));
+}
+
+static void
+show_error_dialog (GtkWidget  *widget,
+                   const char *action_name,
+                   GVariant   *param)
+{
+    ExmErrorDialog *err_dialog;
+    const char *err_text;
+
+    err_text = g_variant_get_string (param, NULL);
+    err_dialog = exm_error_dialog_new (err_text);
+
+    gtk_window_set_modal (GTK_WINDOW (err_dialog), TRUE);
+    gtk_window_set_transient_for (GTK_WINDOW (err_dialog), GTK_WINDOW (widget));
+
+    gtk_window_present (GTK_WINDOW (err_dialog));
+}
+
+static void
+show_error (GtkWidget  *widget,
+            const char *action_name,
+            GVariant   *param)
+{
+    ExmWindow *self;
+    char *error_text;
+    AdwToast *toast;
+
+    self = EXM_WINDOW (widget);
+
+    g_variant_get (param, "s", &error_text);
+
+    toast = adw_toast_new (_("An error occurred."));
+    adw_toast_set_button_label (toast, _("Details"));
+    adw_toast_set_timeout (toast, 5);
+
+    adw_toast_set_action_name (toast, "win.show-error-dialog");
+    adw_toast_set_action_target (toast, "s", error_text);
+
+    adw_toast_overlay_add_toast (self->toast_overlay, toast);
+}
+
+
+static void
+on_error (ExmManager *manager,
+          char       *error_text,
+          ExmWindow  *self)
+{
+    gtk_widget_activate_action (self, "win.show-error", "s", error_text);
 }
 
 static void
@@ -322,12 +389,14 @@ exm_window_class_init (ExmWindowClass *klass)
 
     gtk_widget_class_set_template_from_resource (widget_class, "/com/mattjakeman/ExtensionManager/exm-window.ui");
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, header_bar);
-    gtk_widget_class_bind_template_child (widget_class, ExmWindow, global_toggle);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, installed_page);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, browse_page);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, leaflet);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, main_view);
     gtk_widget_class_bind_template_child (widget_class, ExmWindow, detail_view);
+    gtk_widget_class_bind_template_child (widget_class, ExmWindow, title);
+    gtk_widget_class_bind_template_child (widget_class, ExmWindow, view_stack);
+    gtk_widget_class_bind_template_child (widget_class, ExmWindow, toast_overlay);
 
     // TODO: Refactor ExmWindow into a separate ExmController and supply the
     // necessary actions/methods/etc in there. A reference to this new object can
@@ -338,20 +407,10 @@ exm_window_class_init (ExmWindowClass *klass)
     gtk_widget_class_install_action (widget_class, "ext.open-prefs", "s", extension_open_prefs);
     gtk_widget_class_install_action (widget_class, "win.show-detail", "s", show_view);
     gtk_widget_class_install_action (widget_class, "win.show-main", NULL, show_view);
-    gtk_widget_class_install_action (widget_class, "win.show-release-notes", NULL, show_release_notes);
-}
-
-static void
-version_check_response (GtkDialog *dialog,
-                        gint       response_id,
-                        ExmWindow *self)
-{
-    gtk_window_destroy (GTK_WINDOW (dialog));
-
-    if (response_id == GTK_RESPONSE_YES)
-    {
-        gtk_widget_activate_action (GTK_WIDGET (self), "win.show-release-notes", NULL);
-    }
+    gtk_widget_class_install_action (widget_class, "win.show-upgrade-assistant", NULL, show_upgrade_assistant);
+    gtk_widget_class_install_action (widget_class, "win.show-page", "s", show_page);
+    gtk_widget_class_install_action (widget_class, "win.show-error", "s", show_error);
+    gtk_widget_class_install_action (widget_class, "win.show-error-dialog", "s", show_error_dialog);
 }
 
 static void
@@ -363,30 +422,9 @@ do_version_check (ExmWindow *self)
     settings = g_settings_new (APP_ID);
     version_string = g_settings_get_string (settings, "last-used-version");
 
-    if (strcmp (version_string, APP_VERSION) != 0)
-    {
-        GtkWidget *dialog;
-
-        dialog = gtk_message_dialog_new (GTK_WINDOW (self),
-                                         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                         GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
-                                         _("What's New"));
-
-        gtk_dialog_add_button (GTK_DIALOG (dialog), _("View Release Notes"), GTK_RESPONSE_YES);
-        gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
-
-        // Translators: '%s' = Current version of Extension Manager (e.g. '0.3.0')
-        gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog),
-                                                    _("This is your first time using <b>Extension Manager %s</b>.\nWould you like to see the release notes?"),
-                                                    APP_VERSION);
-
-        g_signal_connect (dialog,
-                          "response",
-                          G_CALLBACK (version_check_response),
-                          self);
-
-        gtk_widget_show (dialog);
-    }
+    // In the future, use this to show a toast or notification when
+    // a new version has been installed. Maybe with rich text release notes?
+    // if (strcmp (version_string, APP_VERSION) != 0) { ... }
 
     g_settings_set_string (settings, "last-used-version", APP_VERSION);
 }
@@ -394,6 +432,8 @@ do_version_check (ExmWindow *self)
 static void
 exm_window_init (ExmWindow *self)
 {
+    gchar *title;
+
     gtk_widget_init_template (GTK_WIDGET (self));
 
     if (IS_DEVEL) {
@@ -401,6 +441,12 @@ exm_window_init (ExmWindow *self)
     }
 
     self->manager = exm_manager_new ();
+    g_signal_connect (self->manager, "error-occurred", on_error, self);
+
+    title = IS_DEVEL ? _("Extension Manager (Development)") : _("Extension Manager");
+
+    gtk_window_set_title (GTK_WINDOW (self), title);
+    adw_view_switcher_title_set_title (self->title, title);
 
     g_object_set (self->installed_page, "manager", self->manager, NULL);
     g_object_set (self->browse_page, "manager", self->manager, NULL);
@@ -412,12 +458,6 @@ exm_window_init (ExmWindow *self)
                             "shell-version",
                             G_BINDING_SYNC_CREATE);
 
-    g_object_bind_property (self->manager,
-                            "extensions-enabled",
-                            self->global_toggle,
-                            "state",
-                            G_BINDING_BIDIRECTIONAL|G_BINDING_SYNC_CREATE);
-
     // Window must be mapped to show version check dialog
     g_signal_connect (self, "map", G_CALLBACK (do_version_check), NULL);
 }
